-- March 2022
-- We obtain de Moebius Equilibrium solution in just one call of IP-OPT, which is warm-started with a configuration having kappa(s) = 2pi et eta(s) = cos (2pi s)

function framePhi(phi) --subroutine
  -- columns are d1 d2 d3
    return {0            ,       0       ,  1,
            math.cos(phi), -math.sin(phi),  0,
            math.sin(phi),  math.cos(phi),  0
    }
    -- so here d3=ex, and (d1,d2) are rotated around ex
end

function genCurvFile(ribbon, filename) --subroutine to export data
  -- export the current state on the disk
  -- columns are:
  -- s Kappa Kappa' Eta Eta' x y z w ;
  -- 1  2      3     4   5   6 7 8 9 ;
  local w = ribbon.width
  local file = io.open(filename, "w")
  local n = ribbon.numberSegments
  local l = 0

  for i=0,n-1,1 do
    local sys = ribbon:getMiniSys(i) -- a b n m : kappa = a s + b and eta = n s + m
    local p = ribbon:getPosStart(i) -- x y z
    file:write(l.."\t"..sys[2].."\t"..sys[1].."\t"..sys[4].."\t"..sys[3].."\t"..p[1].."\t"..p[2].."\t"..p[3].."\t"..w,"\n")
    l=l+ribbon:getLength(i)
  end

  local sys = ribbon:getMiniSys(n-1)
  local p = ribbon:endPos(n-1)
  local Lz = ribbon:getLength(n-1) -- length of last segment
  -- this is to obtain kappa(L) and eta(L)
  file:write(l.."\t"..(sys[1]*Lz+sys[2]).."\t"..sys[1].."\t"..(sys[3]*Lz+sys[4]).."\t"..sys[3].."\t"..(p[1]).."\t"..(p[2]).."\t"..(p[3]).."\t"..w,"\n")
  file:close()
end


-- Main starts here
print()
print(' = * = * Moebius in one shoot * = * = ')
print()
local sribbon
ribbon = SuperRibbon.new()
ribbon.width=0.1 -- width of the ribbon
length = 1.0 -- total contour length set to 1 (adim protocol)
ribbon.areaDensity = 0.0  -- no gravity
ribbon.thickness = 0.005 -- this is used by Mesher (drawing purpose, nothing physical)
ribbon.D = 1.0 -- bending rigidity set to 1 (adim protocol)
ribbon.position = {0.0,0.0,0.0}  -- position of the point at s=0, we put is at origin
ribbon.frame = { 0, 0, 1, 1, 0, 0, 0, 1, 0} -- cosserat frame at s=0 : we choose d1=ey, d2=ez, d3=ex
ribbon.poissonRatio=0.5 -- does not play any role since there is no natural curvature in this case
ribbon.numberSegments = 100 -- number of segments along the ribbon
ribbon:setRibbonStart(2.0*math.pi/length,1.0) -- kappa(s=0) et eta(s=0)
  -- we will start with a seed having kappa(s)= 2 pi and eta(s) = cos(2 pi s)
  -- the idea behind is "twisted circular ring", but it would not be a developpable configuration
  -- so this starting configuration is just a 3D 's-shaped' twisted ribbon
for i=0,ribbon.numberSegments-1,1 do -- segment by segment, we set the starting configuration
  local si = i * length/ribbon.numberSegments -- arc length at start of segment i
  ribbon:setRibbonPiece(i,length/ribbon.numberSegments,0.0,-2*math.pi*math.sin(2*math.pi*si)) -- kappa'(si)=0 and eta'(si)=-2 pi sin (2pi si)
  ribbon:setNaturalCurvature(i,0,0) -- no natural curvature
end
sribbon = MixedSuperRibbon.new(ribbon) -- Use the mixed representation (positions, frame, and curvatures)

mesher = Mesher.new("Moebius_one_shot.obj") -- use e.g. meshlab to open the obj file
mesher.precision=0.005
mesher:setColors({255,255,0},{0,255,255}) -- yellow / cyan
mesher:add(sribbon) -- we draw the seed configuration (that is sent to the minimization algo) [btw it is not a twisted circular ring]

Lk=0.5 --Lk=1/2 is moebius, but you can try other values

options = Options.new()
options.maxIters = 500
options.checkDerivation = false
options.tol = 1.e-9
options.logLevel=3 -- put 3 if you want info on each IP-OPT iteration

local exp = MRFitter.new() -- minimization with mixed ribbon model (so positions are unknowns too)
exp.name="test101"
exp.gravity={0.,0.,0.} -- orientation of gravity
exp.ribbon=sribbon  -- we send the ribbon we created to MRFitter
exp.options=options
exp:attachEnd({0,0,0}, framePhi(2.0*Lk*math.pi)) -- the s=L end must be at origin with its frame rotated around d3(L)=ex
exp:clampRuling(true, true) -- we require eta(0)=0 and eta(L)=0
exp:findEquilibrium() -- IP-OPT is called here (typically takes around 30 iterations, and 2 seconds)

-- some info on the (final) configuration IP-OPT found
print('Minim. Lk='..Lk..' : '..exp.stats.status.." in "..exp.stats.iterations.." iterations with overall error = "..exp.stats.ktt_error)
print("On the final configuration, kappa(s=0) = "..exp.ribbon:getOmega1(0)[2]) -- if there was more segments, should be zero
print("On the final configuration, kappa(s=1/2) = "..exp.ribbon:getOmega1(math.floor(ribbon.numberSegments/2.0))[2])
print("On the final configuration, eta(s=0) = "..exp.ribbon:getEta(0)[2])
print("On the final configuration, eta(s=1/2) = "..exp.ribbon:getEta(math.floor(ribbon.numberSegments/2.0))[2])

-- we export the equilibrium found by IP-OPT on the disk
genCurvFile(exp.ribbon,"lst_z_eta_pos_width_"..string.format("%.3f",exp.ribbon.width)..".txt")

mesher:setColors({255,0,0},{0,0,255}) -- red / blue
mesher:add(exp.ribbon) -- we draw the final configuration

print()
